/* the interleaving is emulated in the system process by permit variables */

`define N 2

/* to emulate program counter */
typedef enum { L1, L2, L3, L4, L5, L6, L7, L8, L9, L10, L11 } loc;

/* the operating system (single processor) */
module system (clk);
input clk;

reg choosing0, choosing1;
reg [0:1] number0, number1;
wire [0:1] num_out0, num_out1;
wire ch_out0, ch_out1;
wire permit0, permit1;
reg st_perm0, st_perm1;

/* the processes are chosen nondeterministically */
assign permit0 = $NDset(@(posedge clk), 0, 1);
assign permit1 = !permit0;

process p0(clk, permit0, choosing0, choosing1, ch_out0, 
                number0, number1, num_out0, 0);
process p1(clk, permit1, choosing0, choosing1, ch_out1, 
                number0, number1, num_out1, 1);

/* only OS can update system memory */
initial begin number0=0; number1=0; choosing0=0; choosing1=0; end
initial st_perm0 = $ND(0,1);
initial st_perm1 = $ND(0,1);
always @(posedge clk) begin
    st_perm0 = permit0; st_perm1 = permit1;
    if (permit0==1) begin number0=num_out0; choosing0=ch_out0; end
    if (permit1==1) begin number1=num_out1; choosing1=ch_out1; end
end

endmodule

/* individual process */
module process (clk, permit, 
                choosing0, choosing1, choosing_out,
                number0, number1, number_out, i);
input clk;
input permit;
input choosing0, choosing1;
output choosing_out;
input [0:1] number0, number1;
output [0:1] number_out;
input [0:1] i;

loc reg pc;
reg [0:1] j;
wire [0:1] numberi, numberj;
wire choosingj;
wire nond_exit;

/* if the process decide to change system wide data */
assign number_out = (pc==L10) ? 0 :
                    (pc==L2) ? ((number0 > number1) ? number0+1 : number1+1) :
                    (i==0) ? number0 : number1;
assign choosing_out = (pc==L1) ? 1 : (pc==L3) ? 0 : 
                      (i==0) ? choosing0 : choosing1;

assign numberi = (i==0) ? number0 : number1;
assign numberj = (j==0) ? number0 : number1;
assign choosingj = (j==0) ? choosing0 : choosing1;
assign nond_exit = $NDset(@(posedge clk), 0, 1);
         
initial begin pc=L1; j=0; end
always @(posedge clk) begin
    if (permit) begin
        case (pc) 
            L1: begin pc=L2; end
            L2: begin pc=L3; end
            L3: begin pc=L4;end
            L4: begin j=0; pc=L5; end
            L5: begin if (j<`N) pc=L6; else pc=L9; end
            L6: begin if (choosingj==1) pc=L6; else pc=L7; end
            L7: begin 
                if ((numberj!=0) && 
                    (numberj<numberi || (numberj==numberi && j<i)))
                    pc=L7; 
                else 
                    pc=L8; 
                end
            L8: begin j=j+1; pc=L5; end
            // critical section
            L9: begin if (nond_exit==1) pc=L10; else pc=L9; end 
            L10: pc=L11;
            L11: begin if (nond_exit==1) pc=L1; else pc=L11; end
        endcase
    end
end

endmodule
