/**CHeaderFile*****************************************************************

  FileName    [baig.h]

  PackageName [baig]

  Synopsis    [Binary AND/INVERTER graph]

  Description [External functions and data strucures of the binary AND/INVERTER graph package]

  SeeAlso     [baigInt.h]

  Author      [Mohammad Awedh]

  Copyright [This file was created at the University of Colorado at
  Boulder.  The University of Colorado at Boulder makes no warranty
  about the suitability of this software for any purpose.  It is
  presented on an AS IS basis.]

  Revision    [$Id: baig.h,v 1.11 2002/09/08 21:42:59 fabio Exp $]


******************************************************************************/

#ifndef _BAIG
#define _BAIG

#include "bdd.h"

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#define bAigInvertBit      0x1                        /* This bit is set to indicate inverted edge*/

#define bAigArrayMaxSize   (1 << 27)  /* Maximum size of the node array */

#define ZERO_NODE_INDEX    0                          /* Reserve this index for constant Zero node*/
#define bAig_Zero          (ZERO_NODE_INDEX)          /* Constatnt Zero node */
#define bAig_One           (bAig_Not(bAig_Zero))      /* Constatnt One node  */

#define bAig_NULL          2

#define bAigNodeSize       4                          /* Array is used to implement the bAig graph.
                                                         Each node occupies 4 array elements. */
#define bAigFirstNodeIndex 4                          /* The first node index. */

#define bAig_HashTableSize 1023

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

typedef char			 nameType_t;
typedef int                      bAigEdge_t;
typedef struct bAigManagerStruct bAig_Manager_t;

/*---------------------------------------------------------------------------*/
/* Structure declarations                                                    */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/


/**Macro***********************************************************************

  Synopsis     [Performs the 'Not' Logical operator]

  Description  [This macro inverts the edge going to a node which corresponds 
                to NOT operator]

  SideEffects [none]

  SeeAlso      []

******************************************************************************/

#define bAig_Not(node) (                         \
        (bAigEdge_t) ((node) ^ (bAigInvertBit))  \
)


/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Function prototypes                                                       */
/*---------------------------------------------------------------------------*/

EXTERN void bAig_Init(void);
EXTERN void bAig_End(void);
EXTERN bAig_Manager_t * bAig_initAig(int maxSize);
EXTERN void bAig_quit(bAig_Manager_t *manager);
EXTERN void bAig_NodePrint(bAig_Manager_t *manager, bAigEdge_t node);
EXTERN nameType_t * bAig_NodeReadName(bAig_Manager_t *manager, bAigEdge_t node);
EXTERN void bAig_NodeSetName(bAig_Manager_t *manager, bAigEdge_t node, nameType_t *name);
EXTERN int bAig_NodeReadIndexOfRightChild(bAig_Manager_t *manager, bAigEdge_t nodeIndex);
EXTERN bAigEdge_t bAig_NodeReadIndexOfLeftChild(bAig_Manager_t *manager, bAigEdge_t nodeIndex);
EXTERN bAigEdge_t bAig_And(bAig_Manager_t *manager, bAigEdge_t nodeIndex1, bAigEdge_t nodeIndex2);
EXTERN bAigEdge_t bAig_Or(bAig_Manager_t *manager, bAigEdge_t nodeIndex1, bAigEdge_t nodeIndex2);
EXTERN bAigEdge_t bAig_Xor(bAig_Manager_t *manager, bAigEdge_t nodeIndex1, bAigEdge_t nodeIndex2);
EXTERN bAigEdge_t bAig_Eq(bAig_Manager_t *manager, bAigEdge_t nodeIndex1, bAigEdge_t nodeIndex2);
EXTERN bAigEdge_t bAig_Then(bAig_Manager_t *manager, bAigEdge_t nodeIndex1, bAigEdge_t nodeIndex2);
EXTERN bAigEdge_t bAig_CreateNode(bAig_Manager_t *manager, bAigEdge_t nodeIndex1, bAigEdge_t nodeIndex2);
EXTERN bAigEdge_t bAig_CreateVarNode(bAig_Manager_t *manager, nameType_t *name);
EXTERN int bAig_isVarNode(bAig_Manager_t *manager, bAigEdge_t node);
EXTERN bAigEdge_t bAig_bddtobAig(bAig_Manager_t *bAigManager, bdd_t *fn);
EXTERN int bAig_PrintDot(FILE *fp, bAig_Manager_t *manager);

/**AutomaticEnd***************************************************************/

#endif /* _BAIG */
