/**CHeaderFile*****************************************************************

  FileName    [baigInt.h]

  PackageName [baig]

  Synopsis    []

  Description [Internal data structures of the bAig package.]

  SeeAlso     []

  Author      [Mohammad Awedh]

  Copyright [ This file was created at the University of Colorado at
  Boulder.  The University of Colorado at Boulder makes no warranty
  about the suitability of this software for any purpose.  It is
  presented on an AS IS basis.]

  Revision    [$Id: baigInt.h,v 1.8 2002/09/12 22:11:44 awedh Exp $]

******************************************************************************/

#ifndef _bAIGINT
#define _bAIGINT

#include "util.h"
#include "st.h"
#include "array.h"
#include "ntk.h"
#include "baig.h"

/*---------------------------------------------------------------------------*/
/* Constant declarations                                                     */
/*---------------------------------------------------------------------------*/

/* Each node occupies 4 location.  The following describes these locations */

#define  bAigLeft     0   /* Index of the left child */ 
#define  bAigRight    1   /* Index of the right child */ 
#define  bAigNext     2   /* Index of the next node in the Hash Table. */  
#define  bAigRefCount 3   /* Index of reference count for this node.*/

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Structure declarations                                                    */
/*---------------------------------------------------------------------------*/


/**Struct**********************************************************************

  Synopsis    [And/Inverter graph manager.]

  Description []

******************************************************************************/

struct bAigManagerStruct {
  bAigEdge_t *NodesArray;       /* contains pointers to nodes of type
				   bAig_Node_t */
  int        nodesArraySize;    /* contains the current number of nodes */
  int        maxNodesArraySize; /* Nodes Array maximum size */

  bAigEdge_t *HashTable;        /* Hash table */
  st_table   *SymbolTable;      /* Symbol table to store variables and their
				   indices */
  char       **nameList;        /* This table contains the name of each node.
				   nodeID is used to access the node's name. */
  array_t    *mVarList;         /* List of mVarId of each variable */ 
  array_t    *bVarList;         /* List of bVardId (bAig_t) of the binary value
				   encoding for multi-valued variables. */
};


/*---------------------------------------------------------------------------*/
/* Variable declarations                                                     */
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* Macro declarations                                                        */
/*---------------------------------------------------------------------------*/


/**Macro***********************************************************************

  Synopsis     [Returns the non inverted edge.]

  Description  [This macro returns the non inverted edge.  It clears the least 
  significant bit of the edge if it is set.]

  SideEffects  [none]

  SeeAlso      []

******************************************************************************/

#define bAig_NonInvertedEdge(edge) ( \
	(edge) & ~(bAigInvertBit)    \
)

/**Macro***********************************************************************

  Synopsis     [Returns true if the edge is inverted.]

  Description  []

  SideEffects [none] 

  SeeAlso      []

******************************************************************************/

#define bAig_IsInverted(edge) (   \
	(edge) & (bAigInvertBit)  \
)

/**Macro***********************************************************************

  Synopsis    []

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

#define rightChild(node)(\
manager->NodesArray[bAig_NonInvertedEdge(node)+bAigRight]\
)

/**Macro***********************************************************************

  Synopsis    []

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

#define leftChild(node)(\
manager->NodesArray[bAig_NonInvertedEdge(node)+bAigLeft]\
)

/**Macro***********************************************************************

  Synopsis    []

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

#define refCount(node)(\
manager->NodesArray[bAig_NonInvertedEdge(node)+bAigRefCount]\
)


/**Macro***********************************************************************

  Synopsis    []

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

#define next(node)(\
manager->NodesArray[bAig_NonInvertedEdge(node)+bAigNext]\
)

/**Macro***********************************************************************

  Synopsis    [The node ID]

  Description [Node ID is a unique number for each node and can be found by 
              dividing the edge going to this node by bAigNodeSize.]

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

#define bAigNodeID(edge)(\
bAig_NonInvertedEdge(edge)>>2 \
)

/**Macro***********************************************************************

  Synopsis    [Hash function for the hash table.]

  Description []

  SideEffects [none]

  SeeAlso     []

******************************************************************************/

#define HashTableFunction(left, right) \
( (bAigEdge_t)\
((( (bAigEdge_t) (left)) << 3) ^ \
(( (bAigEdge_t) (right)) << 3)) % bAig_HashTableSize \
)


/**AutomaticStart*************************************************************/

/*---------------------------------------------------------------------------*/
/* Function prototypes                                                       */
/*---------------------------------------------------------------------------*/

EXTERN bAigEdge_t bAigCreateAndNode(bAig_Manager_t *manager, bAigEdge_t node1, bAigEdge_t node2);

/**AutomaticEnd***************************************************************/

#endif /* _bAIGINT */

































